 #include <DFRobot_DS3231M.h>
#include "Servo.h"
#include <Arduino.h>
#include <U8g2lib.h>
#include <Wire.h>

#define KEY 9

/********请将事件写入数组中,事件类型写入枚举和字符数组中******/
uint8_t dayThings[][3] = {  //每日事件  闹钟2  {某时,某分,事件类型}
  {16,1,0},
  {16,2,1}
};

uint8_t monthThings[][3] = {  //每月事件  闹钟1  {某日,某时,事件类型}
  {21,16,2},
  {1,0,3}
};

const char* Things[4] = {  //事件类型
  "clock In",
  "clock Out",
  "visit",
  "travel"
};

/*************************请在setup中设置当前时间****************************/
/************************************************************************/


volatile  int8_t alarmFlag = 0;
uint8_t remindFlag = 0;

uint8_t dayCount;     //每天事件个数
uint8_t monthCount;   //每月事件个数
uint8_t dayCurrentThings;       //每天即将发生事件
uint8_t monthCurrentThings;     //每月即将发生事件
int8_t pos = 60;



DFRobot_DS3231M rtc;
Servo myservo;
U8G2_SSD1306_128X32_UNIVISION_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);  //  M0/ESP32/ESP8266/mega2560/Uno/Leonardo

void setup(void)
{
  dayCount = sizeof(dayThings) / sizeof(dayThings[0]);
  monthCount = sizeof(monthThings) / sizeof(monthThings[0]);
  bubble_sort(dayThings, dayCount);        //将每日事件排序
  bubble_sort(monthThings, monthCount);    //将每月事件排序
  
  while(rtc.begin() != true){
    delay(1000);
  }
  rtc.writeSqwPinMode(eDS3231M_OFF);
  
  rtc.enAbleAlarm1Int();
  rtc.enAbleAlarm2Int();

  if(rtc.lostPower()){  //如果掉电则修改时间
    rtc.setYear(20);
    rtc.setMonth(2);
    rtc.setDate(28);
    rtc.setHour(16,e24hours);
    rtc.setMinute(0);
    rtc.setSecond(58);
    rtc.adjust();
  }
  
  initAlarm();   //将近期事件写入闹钟
  
  pinMode(KEY, INPUT);
  
  myservo.attach(11);
  myservo.write(90);

  u8g2.begin();
  u8g2.setFontPosTop();

}

void loop() {
  if(remindFlag == 1){     //触发提醒
    myservo.write(90+pos); //左右摆动
    pos = -pos;
  }else{                   //正常显示时间
    rtc.getNowTime();

    u8g2.clearBuffer();
    u8g2.setFont(u8g2_font_ncenB18_tf);
    u8g2.drawStr(0,7,String(rtc.hour()).c_str());   //时
    u8g2.drawStr(30,7,":");
    u8g2.drawStr(40,7,String(rtc.minute()).c_str());  //分

    u8g2.setFont(u8g2_font_6x13_tf);

    u8g2.drawStr(80,0,String(rtc.month()).c_str());   //月
    u8g2.drawStr(95,0,"/");
    u8g2.drawStr(105,0,String(rtc.day()).c_str());   //日

    u8g2.drawStr(70,20,String(rtc.second()).c_str());    //秒

    u8g2.drawStr(90,20,rtc.getDayOfTheWeek());   //星期

    u8g2.sendBuffer();
  }
  alarmFlag = rtc.isAlarm();  //读取闹钟
  if(alarmFlag){
    if(alarmFlag == 2){  //闹钟2显示事件
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_ncenB18_tf);
      u8g2.drawStr(0,6,Things[dayThings[dayCurrentThings][2]]);
      u8g2.sendBuffer();
      if(++dayCurrentThings > dayCount) dayCurrentThings = 0;
      rtc.setAlarm(eMinutesHoursMatch,1,dayThings[dayCurrentThings][0],e24hours,dayThings[dayCurrentThings][1],0);  //设置下一个闹钟
    }
    if(alarmFlag == 1){  //闹钟1显示事件
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_ncenB18_tf);
      u8g2.drawStr(0,6,Things[monthThings[monthCurrentThings][2]]);
      u8g2.sendBuffer();
      if(++monthCurrentThings > monthCount) monthCurrentThings = 0;
      rtc.setAlarm(eSecondsMinutesHoursDateMatch,monthThings[monthCurrentThings][0],monthThings[monthCurrentThings][1],e24hours,0,0);
    }
    alarmFlag = 0;
    remindFlag = 1;  //打开提醒标志
    rtc.clearAlarm();
  }
  if(digitalRead(KEY) == 1){  //按键按下  需要按下1秒
    remindFlag = 0; //停止提醒
    myservo.write(90);
  }
  delay(1000);
}

void bubble_sort(uint8_t arr[][3], uint8_t len){   //事件排序
  uint8_t i, j, k, temp;
  for (i = 0; i < len - 1; i++){
    for (j = 0; j < len - 1 - i; j++){
      if (arr[j][0] > arr[j + 1][0]){
        for(k = 0; k < 3; k++){
          temp = arr[j][k];
          arr[j][k] = arr[j + 1][k];
          arr[j + 1][k] = temp;
        }
      }else if(arr[j][0] == arr[j + 1][0]){
        if(arr[j][1] > arr[j + 1][1]){
          for(k = 0; k < 3; k++){
            temp = arr[j][k];
            arr[j][k] = arr[j + 1][k];
            arr[j + 1][k] = temp;
          }
        }
      }
    }
  }
}



void initAlarm(){   //将近期事件写入闹钟
  uint8_t i;
  
  rtc.setAlarm(eMinutesHoursMatch,1,dayThings[0][0],e24hours,dayThings[0][1],0);
  dayCurrentThings = 0;
  rtc.setAlarm(eSecondsMinutesHoursDateMatch,monthThings[0][0],monthThings[0][1],e24hours,0,0);
  monthCurrentThings = 0;
  
  rtc.getNowTime();
  for(i = 0; i < dayCount; i++){
    if(rtc.hour() < dayThings[i][0]){
      rtc.setAlarm(eMinutesHoursMatch,1,dayThings[i][0],e24hours,dayThings[i][1],0);  //设置闹钟
      dayCurrentThings = i;  //记录事件号
      break;
    }else if(rtc.hour() == dayThings[i][0]){
      if(rtc.minute() < dayThings[i][1]){
        rtc.setAlarm(eMinutesHoursMatch,1,dayThings[i][0],e24hours,dayThings[i][1],0);  //设置闹钟
        dayCurrentThings = i;  //记录事件号
        break;
      }
    }
  }
  
  for(i = 0; i < monthCount; i++){
    if(rtc.day() < monthThings[i][0]){
      rtc.setAlarm(eSecondsMinutesHoursDateMatch,monthThings[i][0],monthThings[i][1],e24hours,0,0);
      monthCurrentThings = i;
      break;
    }else if(rtc.day() == monthThings[i][0]){
      if(rtc.hour() < monthThings[i][1]){
        rtc.setAlarm(eSecondsMinutesHoursDateMatch,monthThings[i][0],monthThings[i][1],e24hours,0,0);
        monthCurrentThings = i;
        break;
      }
    }
  }
}
